@@ -10,7 +10,6 @@ from __future__ import annotations
 
 from typing import Literal
 
-import cv2
 import numpy as np
 import scipy.ndimage as spi
 import scipy.spatial as spt
@@ -259,14 +258,10 @@ def get_centroid_fourier(data: np.ndarray) -> tuple[float, float]:
 
     row = (np.arctan(b / a) + rphi) * (rows - 1) / (2 * np.pi) + 1
     col = (np.arctan(d / c) + cphi) * (cols - 1) / (2 * np.pi) + 1
-    try:
-        row = int(row)
-    except ma.MaskError:
-        row = np.nan
-    try:
-        col = int(col)
-    except ma.MaskError:
-        col = np.nan
+
+    row = np.nan if row is np.ma.masked else row
+    col = np.nan if col is np.ma.masked else col
+
     return row, col
 
 
@@ -720,6 +715,14 @@ def find_blobs_opencv(
     Returns:
         Coordinates of blobs
     """
+
+    # Note:
+    # Importing OpenCV inside the function in order to eventually raise an ImportError
+    # when the function is called and OpenCV is not installed. This error will be
+    # handled by DataLab and the user will be informed that OpenCV is required to use
+    # this function.
+    import cv2  # pylint: disable=import-outside-toplevel
+
     params = cv2.SimpleBlobDetector_Params()
     if min_threshold is not None:
         params.minThreshold = min_threshold
