@@ -25,7 +25,7 @@ from typing import TYPE_CHECKING
 
 import scipy.integrate as spt
 from guidata.configtools import get_icon, get_image_file_path
-from guidata.qthelpers import create_action, is_dark_mode
+from guidata.qthelpers import create_action, is_dark_theme
 from guidata.widgets.dockable import DockableWidget
 from plotpy.constants import PlotType
 from plotpy.items import CurveItem
@@ -64,9 +64,12 @@ if TYPE_CHECKING:
 
 def fwhm_info(x, y):
     """Return FWHM information string"""
-    with warnings.catch_warnings(record=True) as w:
-        x0, _y0, x1, _y1 = fwhm((x, y), "zero-crossing")
-        wstr = " ⚠️" if w else ""
+    try:
+        with warnings.catch_warnings(record=True) as w:
+            x0, _y0, x1, _y1 = fwhm((x, y), "zero-crossing")
+            wstr = " ⚠️" if w else ""
+    except ValueError:
+        return "🛑"
     return f"{x1 - x0:g}{wstr}"
 
 
@@ -337,7 +340,7 @@ class DockablePlotWidget(DockableWidget):
 
     def update_color_mode(self) -> None:
         """Update plot widget styles according to application color mode"""
-        if is_dark_mode():
+        if is_dark_theme():
             palette = QApplication.instance().palette()
         else:
             palette = QG.QPalette(QC.Qt.white)
