@@ -28,11 +28,12 @@ import guidata.dataset as gds
 import numpy as np
 import scipy.ndimage as spi
 import scipy.signal as sps
+from guidata import qthelpers as guidata_qth
 from guidata.configtools import get_icon
-from guidata.qthelpers import add_actions, create_action, win32_fix_title_bar_background
+from guidata.qthelpers import add_actions, create_action
 from guidata.widgets.console import DockableConsole
+from plotpy import config as plotpy_config
 from plotpy.builder import make
-from plotpy.config import set_plotpy_dark_mode
 from plotpy.constants import PlotType
 from qtpy import QtCore as QC
 from qtpy import QtGui as QG
@@ -176,7 +177,7 @@ class CDLMainWindow(QW.QMainWindow, AbstractCDLControl, metaclass=CDLMainWindowM
         self.view_menu: QW.QMenu | None = None
         self.help_menu: QW.QMenu | None = None
 
-        self.__update_color_mode()
+        self.__update_color_mode(startup=True)
 
         self.__is_modified = False
         self.set_modified(False)
@@ -842,9 +843,9 @@ class CDLMainWindow(QW.QMainWindow, AbstractCDLControl, metaclass=CDLMainWindowM
         )
         self.showfirstonly_action = create_action(
             self,
-            _("Show first only"),
+            _("Show first object only"),
             icon=get_icon("show_first.svg"),
-            tip=_("Show only the first selected object"),
+            tip=_("Show only the first selected object (signal or image)"),
             toggled=self.toggle_show_first_only,
         )
         self.showlabel_action = create_action(
@@ -1590,22 +1591,35 @@ class CDLMainWindow(QW.QMainWindow, AbstractCDLControl, metaclass=CDLMainWindowM
               <p>{adv_conf}""",
         )
 
-    def __update_color_mode(self) -> None:
-        """Update color mode"""
-        color_mode = Conf.main.color_mode.get()
-        if color_mode != "auto":
-            set_plotpy_dark_mode(color_mode == "dark")
-            if self.docks is not None:
-                for dock in self.docks.values():
-                    widget = dock.widget()
-                    if isinstance(widget, DockablePlotWidget):
-                        widget.update_color_mode()
-            if self.console is not None:
-                self.console.update_color_mode()
-                self.console.clear()
-            if self.macropanel is not None:
-                self.macropanel.update_color_mode()
-        win32_fix_title_bar_background(self)
+    def __update_color_mode(self, startup: bool = False) -> None:
+        """Update color mode
+
+        Args:
+            startup: True if method is called during application startup (in that case,
+             color theme is applied only if mode != "auto")
+        """
+        mode = Conf.main.color_mode.get()
+        if startup and mode == "auto":
+            guidata_qth.win32_fix_title_bar_background(self)
+            return
+
+        # Prevent Qt from refreshing the window when changing the color mode:
+        self.setUpdatesEnabled(False)
+
+        plotpy_config.set_plotpy_color_mode(mode)
+
+        if self.console is not None:
+            self.console.update_color_mode()
+        if self.macropanel is not None:
+            self.macropanel.update_color_mode()
+        if self.docks is not None:
+            for dock in self.docks.values():
+                widget = dock.widget()
+                if isinstance(widget, DockablePlotWidget):
+                    widget.update_color_mode()
+
+        # Allow Qt to refresh the window:
+        self.setUpdatesEnabled(True)
 
     def __edit_settings(self) -> None:
         """Edit settings"""
