@@ -31,8 +31,17 @@ def test_main_app():
         except ValueError:
             pass
 
-        # Add signals to signal panel
         panel = win.signalpanel
+
+        # Create new groups
+        panel.add_group("Group 1")
+        panel.add_group("Group 2")
+        # Rename group
+        panel.objview.select_groups([2])
+        panel.rename_group("Group xxx")
+        panel.remove_object(force=True)
+
+        # Add signals to signal panel
         sig1 = create_paracetamol_signal(500)
         panel.add_object(sig1)
         panel.processor.compute_derivative()
