@@ -10,7 +10,6 @@ executed before running any tests.
 
 import os
 
-import cv2
 import guidata
 import h5py
 import numpy
@@ -47,9 +46,14 @@ def pytest_report_header(config):  # pylint: disable=unused-argument
         f"PythonQwt {qwt.__version__}, "
         f"{qtpy.API_NAME} {qtbindings_version} [Qt version: {qtpy.QT_VERSION}]",
         f"NumPy {numpy.__version__}, SciPy {scipy.__version__}, "
-        f"h5py {h5py.__version__}, "
-        f"scikit-image {skimage.__version__}, OpenCV {cv2.__version__}",
+        f"h5py {h5py.__version__}, scikit-image {skimage.__version__}",
     ]
+    try:
+        import cv2  # pylint: disable=import-outside-toplevel
+
+        infolist[-1] += f", OpenCV {cv2.__version__}"
+    except ImportError:
+        pass
     for vname in ("CDL_DATA", "PYTHONPATH", "DEBUG"):
         value = os.environ.get(vname, "")
         if value:
