@@ -17,6 +17,7 @@ import guidata.dataset as gds
 import numpy as np
 
 import cdl.obj
+from cdl.algorithms.datatypes import is_integer_dtype
 from cdl.config import _
 from cdl.utils.tests import get_test_fnames
 
@@ -215,7 +216,7 @@ def create_2d_random(
         2D data
     """
     rng = np.random.default_rng(seed)
-    amp = np.iinfo(dtype).max * level
+    amp = (np.iinfo(dtype).max if is_integer_dtype(dtype) else 1.0) * level
     return np.array(rng.random((size, size)) * amp, dtype=dtype)
 
 
@@ -543,7 +544,7 @@ def create_noisygauss_image(
     p = __set_default_size_dtype(p)
     p.title = "Test image (noisy 2D Gaussian)" if p.title is None else p.title
     dtype = p.dtype.value
-    size = p.width
+    size = max(p.height, p.width)
     obj = cdl.obj.create_image_from_param(p)
     if center is None:
         # Default center
@@ -555,6 +556,11 @@ def create_noisygauss_image(
         obj.data += create_2d_random(size, dtype, level)
     if add_annotations:
         obj.add_annotations_from_file(get_test_fnames("annotations.json")[0])
+    if p.height != p.width:
+        if p.height > p.width:
+            obj.data = obj.data[:, p.width // 2 : -p.width // 2]
+        else:
+            obj.data = obj.data[p.height // 2 : -p.height // 2, :]
     return obj
 
 
