@@ -8,9 +8,10 @@ ROI image parameters unit test.
 # guitest: show
 
 import guidata.dataset as gds
+import numpy as np
 from guidata.qthelpers import qt_app_context
 
-from cdl.core.model.image import ROI2DParam, RoiDataGeometries
+from cdl.core.model.image import ROI2DParam
 from cdl.env import execenv
 
 
@@ -18,12 +19,15 @@ def test_roi_param_interactive():
     """ROI parameters interactive test."""
     with qt_app_context():
         p_circ = ROI2DParam("Circular")
-        p_circ.geometry = RoiDataGeometries.CIRCLE
+        p_circ.geometry = "circle"
         p_circ.xc, p_circ.yc, p_circ.r = 100, 200, 50
         p_rect = ROI2DParam("Rectangular")
-        p_rect.geometry = RoiDataGeometries.RECTANGLE
-        p_rect.xr0, p_rect.yr0, p_rect.xr1, p_rect.yr1 = 50, 150, 150, 250
-        params = [p_circ, p_rect]
+        p_rect.geometry = "rectangle"
+        p_rect.x0, p_rect.y0, p_rect.dx, p_rect.dy = 50, 150, 150, 250
+        p_poly = ROI2DParam("Polygonal")
+        p_poly.geometry = "polygon"
+        p_poly.points = np.array([[50, 150], [150, 150], [150, 250], [50, 250]])
+        params = [p_circ, p_rect, p_poly]
         group = gds.DataSetGroup(params, title="ROI Parameters")
         if group.edit():
             for param in params:
