@@ -8,6 +8,8 @@ from __future__ import annotations
 
 import os
 import os.path as osp
+import warnings
+from typing import Any
 
 from guidata.userconfig import NoDefault, UserConfig
 
@@ -77,49 +79,28 @@ class Section:
 class Option:
     """Configuration option handler"""
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.section = None
         self.option = None
 
-    def get(self, default=NoDefault):
+    def get(self, default=NoDefault) -> Any:
         """Get configuration option value"""
         return CONF.get(self.section, self.option, default)
 
-    def set(self, value):
+    def set(self, value: Any) -> None:
         """Set configuration option value"""
         CONF.set(self.section, self.option, value)
 
-    def remove(self):
+    def remove(self) -> None:
         """Remove configuration option"""
         # No use case for this method yet (quite dangerous!)
         CONF.remove_option(self.section, self.option)
 
 
-class EnumOption(Option):
-    """Enum configuration option handler"""
-
-    def __init__(self, choices: list[str | int | float]) -> None:
-        super().__init__()
-        self.choices = choices
-
-    def get(self, default=NoDefault) -> str | int | float:
-        """Get configuration option value"""
-        value = super().get(default)
-        if value not in self.choices:
-            raise ValueError(f"Invalid configuration option value {value}")
-        return value
-
-    def set(self, value: str | int | float) -> None:
-        """Set configuration option value"""
-        if value not in self.choices:
-            raise ValueError(f"Invalid configuration option value {value}")
-        super().set(value)
-
-
 class ConfigPathOption(Option):
     """Configuration file path configuration option handler"""
 
-    def get(self, default=NoDefault):
+    def get(self, default=NoDefault) -> str:
         """Get configuration file path from configuration"""
         if default is NoDefault:
             default = ""
@@ -132,7 +113,7 @@ class ConfigPathOption(Option):
 class WorkingDirOption(Option):
     """Working directory configuration option handler"""
 
-    def get(self, default=NoDefault):
+    def get(self, default=NoDefault) -> str:
         """Get working directory from configuration"""
         if default is NoDefault:
             default = ""
@@ -141,7 +122,7 @@ class WorkingDirOption(Option):
             return path
         return ""
 
-    def set(self, value):
+    def set(self, value: str) -> None:
         """Set working directory in configuration"""
         if not osp.isdir(value):
             value = osp.dirname(value)
@@ -151,6 +132,38 @@ class WorkingDirOption(Option):
         super().set(value)
 
 
+class EnumOption(Option):
+    """Enumeration option handler"""
+
+    def __init__(self, values: list[Any], default: Any = NoDefault) -> None:
+        super().__init__()
+        if default is NoDefault:
+            default = values[0]
+        self.values = values
+        self.default = default
+
+    def get(self, default: Any = NoDefault) -> Any:
+        """Get configuration option value"""
+        value = super().get(default)
+        if value not in self.values:
+            # Only show a warning here, as the configuration file may be edited manually
+            warnings.warn(
+                f"Invalid value {value} for option {self.option}, "
+                f"expected {self.values}"
+            )
+            return self.default
+        return value
+
+    def set(self, value: Any) -> None:
+        """Set configuration option value"""
+        if value not in self.values:
+            raise ValueError(
+                f"Invalid value {value} for option {self.option}, "
+                f"expected {self.values}"
+            )
+        super().set(value)
+
+
 class SectionMeta(type):
     """Configuration metaclass"""
 
